@@ -1,7 +1,7 @@
 #!/usr/bin/python3
-# Copyright (c) 2017 David Steele <dsteele@gmail.com>
+# Copyright (c) 2017-2019 David Steele <dsteele@gmail.com>
 #
-# SPDX-License-Identifier: GPL-2+
+# SPDX-License-Identifier: GPL-2.0-or-later
 # License-Filename: LICENSE
 
 #
@@ -11,14 +11,15 @@
 # or later
 #
 
-import logging
-import NetworkManager as nm
 import argparse
 import dbus
+from functools import wraps
+import getpass
+import logging
+import NetworkManager as nm
+import pprint
 import sys
 import uuid
-import getpass
-from functools import wraps
 
 if __name__ == '__main__':
     import os
@@ -26,20 +27,26 @@ if __name__ == '__main__':
     parentdir = '/'.join(fullpath.split('/')[:-2])
     sys.path.insert(0, parentdir)
 
-from comitup import iwscan
+from comitup import iwscan  # noqa
 
 
-import pprint
 pp = pprint.PrettyPrinter(indent=4)
 
 log = logging.getLogger('comitup')
 
+this_module = sys.modules[__name__]
+for key in nm.__dict__:
+    if key.startswith("NM_"):
+        setattr(this_module, key, nm.__dict__[key])
+
 
 def initialize():
     nm.Settings.ReloadConnections()
 
 
 def nm_state():
+
+    log.debug("Calling nm.NetworkManager.State()")
     return nm.NetworkManager.State
 
 
@@ -58,12 +65,19 @@ def none_on_exception(*exceptions):
     return _none_on_exception
 
 
+device_list = None
 def get_devices():
-    try:
-        return nm.NetworkManager.GetDevices()
-    except TypeError:
-        # NetworkManager is gone for some reason. Bail big time.
-        sys.exit(1)
+    global device_list
+
+    if not device_list:
+        try:
+            log.debug("Calling nm.GetDevices()")
+            device_list = nm.NetworkManager.GetDevices()
+        except TypeError:
+            # NetworkManager is gone for some reason. Bail big time.
+            sys.exit(1)
+
+    return device_list
 
 
 def device_name(device):
@@ -84,19 +98,37 @@ def get_wifi_device(index=0):
 
 
 def get_device_path(device):
+    log.debug("Getting specific device for path")
     return device.SpecificDevice().object_path
 
 
 def disconnect(device):
     try:
+        log.debug("Calling disconnect")
         device.Disconnect()
-    except:
+    except:   # noqa
         log.debug("Error received in disconnect")
 
 
+settings_cache = {}
+def get_connection_settings(connection):
+    global settings_cache
+
+    if connection.uuid not in settings_cache:
+        log.debug("Not in cache")
+        settings_cache[connection.uuid] = connection.GetSettings()
+
+    return settings_cache[connection.uuid]
+
+
 def get_device_settings(device):
-    connection = device.ActiveConnection
-    return connection.Connection.GetSettings()
+    try:
+        connection = device.ActiveConnection
+    except NetworkManager.ObjectVanished:
+        sys.exit(1)
+
+    log.debug("Getting Connection settings")
+    return get_connection_settings(connection.Connection)
 
 
 @none_on_exception(AttributeError)
@@ -110,12 +142,14 @@ def get_active_ip(device):
 
 
 def get_all_connections():
+    log.debug("Calling nm.ListConnections()")
     return [x for x in nm.Settings.ListConnections()]
 
 
 @none_on_exception(AttributeError, KeyError)
 def get_ssid_from_connection(connection):
-    settings = connection.GetSettings()
+    log.debug("Calling GetSettings")
+    settings = get_connection_settings(connection)
 
     return settings['802-11-wireless']['ssid']
 
@@ -139,17 +173,20 @@ def del_connection_by_ssid(name):
 def activate_connection_by_ssid(ssid, device, path='/'):
     connection = get_connection_by_ssid(ssid)
 
+    log.debug("Calling nm.ActivateConnection()")
     nm.NetworkManager.ActivateConnection(connection, device, path)
 
 
 def deactivate_connection(device):
     connection = device.ActiveConnection
     if connection:
+        log.debug("Calling nm.DeactivateConnection()")
         nm.NetworkManager.DeactivateConnection(connection)
 
 
 @none_on_exception(AttributeError)
 def get_access_points(device):
+    log.debug("Calling GetAllAccessPoints()")
     return device.SpecificDevice().GetAllAccessPoints()
 
 
@@ -187,7 +224,8 @@ def get_candidate_connections(device):
     candidates = []
 
     for conn in get_all_connections():
-        settings = conn.GetSettings()
+        log.debug("Getting settings 2")
+        settings = get_connection_settings(conn)
         ssid = get_ssid_from_connection(conn)
 
         try:
@@ -199,29 +237,40 @@ def get_candidate_connections(device):
         except KeyError:
             log.debug("Unexpected connection format for %s" % ssid)
 
-    points = [x.Ssid for x in get_access_points(device)]
-    iwpoints = [x['ssid'] for x in iwscan.candidates()]
+#     # kicknm
+#     get_access_points(device)
+#     iwscan.candidates()
 
-    return list(set(candidates) & (set(points) | set(iwpoints)))
+    log.debug("candidates: %s" % candidates)
+
+    return candidates
 
 
-def make_hotspot(name='comitup', device=None):
+def make_hotspot(name='comitup', device=None, password="", hash="0000"):
+    comitup_uuid = str(uuid.uuid4())
     settings = {
         'connection':
         {
             'type': '802-11-wireless',
-            'uuid': str(uuid.uuid4()),
-            'id': name,
+            'uuid': comitup_uuid,
+            'id': "{0}-{1}".format(name, hash),
             'autoconnect': False,
         },
         '802-11-wireless':
         {
             'mode': 'ap',
             'ssid': name,
+            'band': "bg",
         },
         'ipv4':
         {
-            'method': 'shared',
+            'method': 'manual',
+            'address-data': [
+                {
+                    'address': '10.42.0.1',
+                    'prefix': 24,
+                }
+            ]
         },
         'ipv6':
         {
@@ -232,6 +281,12 @@ def make_hotspot(name='comitup', device=None):
     if device:
         settings['connection']['interface-name'] = device_name(device)
 
+    if password and len(password) >= 8:
+        settings['802-11-wireless-security'] = {}
+        settings['802-11-wireless-security']['key-mgmt'] = "wpa-psk"
+        settings['802-11-wireless-security']['psk'] = password
+
+    log.debug("Calling nm.AddConnection()")
     nm.Settings.AddConnection(settings)
 
 
@@ -273,6 +328,7 @@ def make_connection_for(ssid, password=None, interface=None):
             'psk': password,
         })
 
+    log.debug("Calling nm.AddConnection()")
     nm.Settings.AddConnection(settings)
 
 
@@ -300,7 +356,7 @@ def do_listaccess(arg):
     try:
         import tabulate
         print(tabulate.tabulate(bypwr, headers=hdrs))
-    except:
+    except:    # noqa
         for entry in bypwr:
             print(entry)
 
