@@ -1,7 +1,7 @@
 #!/usr/bin/python3
-# Copyright (c) 2017 David Steele <dsteele@gmail.com>
+# Copyright (c) 2017-2019 David Steele <dsteele@gmail.com>
 #
-# SPDX-License-Identifier: GPL-2+
+# SPDX-License-Identifier: GPL-2.0-or-later
 # License-Filename: LICENSE
 
 #
@@ -11,12 +11,14 @@
 # or later
 #
 
+from dbus.exceptions import DBusException
+from functools import wraps
+import hashlib
 import logging
 import time
-from functools import wraps
-from dbus.exceptions import DBusException
-from comitup import iwscan
 
+from comitup import iwscan
+from comitup import wpa
 
 from gi.repository.GLib import MainLoop, timeout_add
 if __name__ == '__main__':
@@ -99,18 +101,12 @@ def hotspot_start():
 
     hs_ssid = dns_to_conn(dns_names[0])
 
-    # if we are in two-wifi device mode, skip the reconnect if possible,
-    # to avoid kicking some clients off
     log.debug("states: Calling nm.get_active_ssid()")
     if hs_ssid != nm.get_active_ssid(modemgr.get_state_device('HOTSPOT')):
         mdns.clear_entries()
         conn_list = []
 
-        # tolerate Raspberry Pi 2
-        try:
-            activate_connection(hs_ssid, 'HOTSPOT')
-        except DBusException:
-            log.warn("Error connecting hotspot")
+        activate_connection(hs_ssid, 'HOTSPOT')
     else:
         log.debug("Didn't need to reactivate - already running")
         # the connect callback won't happen - let's 'pass' manually
@@ -134,26 +130,26 @@ def hotspot_pass():
 
 @state_callback
 def hotspot_fail():
-    log.warn("Hotspot mode failure")
+    log.warning("Hotspot mode failure")
     pass
 
 
 @timeout
 def hotspot_timeout():
-
     if iwscan.ap_conn_count() == 0 or modemgr.get_mode() != 'single':
         log.debug('Periodic connection attempt')
 
         dev = modemgr.get_state_device('CONNECTED')
         conn_list = candidate_connections(dev)
         if conn_list:
-            # bug - try the first connection twice
-            set_state('CONNECTING', [conn_list[0], conn_list[0]] + conn_list)
+            set_state('CONNECTING', conn_list)
         else:
-            set_state('CONNECTING')
+            log.info('No candidates - skipping CONNECTING scan')
     else:
         log.info('AP active - skipping CONNECTING scan')
 
+    wpa.check_wpa(modemgr.get_ap_device().Interface)
+
 
 #
 # Connecting state
@@ -174,14 +170,6 @@ def connecting_start():
         log.info('Attempting connection to %s' % conn)
         activate_connection(conn, 'CONNECTING')
     else:
-        # Give NetworkManager a chance to update the access point list
-        try:
-            # todo - clean this up
-            log.debug("states: Calling nm.deactivate_connection()")
-            nm.deactivate_connection(modemgr.get_state_device('CONNECTING'))
-        except DBusException:
-            pass
-        time.sleep(5)
         set_state('HOTSPOT')
 
 
@@ -234,7 +222,7 @@ def connected_pass():
 
 @state_callback
 def connected_fail():
-    log.warn('Connection lost')
+    log.warning('Connection lost')
     set_state('HOTSPOT')
 
 
@@ -242,9 +230,12 @@ def connected_fail():
 def connected_timeout():
     log.debug("states: Calling nm.get_active_ssid()")
     if connection != nm.get_active_ssid(modemgr.get_state_device('CONNECTED')):
-        log.warn("Connection lost on timeout")
+        log.warning("Connection lost on timeout")
         set_state('HOTSPOT')
 
+    if modemgr.get_mode() == modemgr.MULTI_MODE:
+        wpa.check_wpa(modemgr.get_ap_device().Interface)
+
 
 #
 # State Management
@@ -271,22 +262,22 @@ def set_state(state, connections=None, timeout=180):
 
     log.info('Setting state to %s' % state)
 
-    if com_state != 'HOTSPOT':
-        log.debug("states: Calling nm.get_points_ext()")
-        points = nm.get_points_ext(modemgr.get_state_device(com_state))
-
     state_info = state_matrix(state)
 
-    nmmon.set_device_callbacks(state, state_info.pass_fn, state_info.fail_fn)
+    nmmon.init_nmmon()
+    nmmon.enable(
+        modemgr.get_state_device(state),
+        state_info.pass_fn,
+        state_info.fail_fn
+    )
 
     if connections:
         conn_list = connections
 
     state_id += 1
     com_state = state
-    state_info.start_fn()
-
     timeout_add(timeout*1000, state_info.timeout_fn, state_id)
+    state_info.start_fn()
 
 
 def activate_connection(name, state):
@@ -315,30 +306,32 @@ def set_hosts(*args):
     dns_names = args
 
 
-def assure_hotspot(ssid, device):
-    log.debug("states: Calling nm.get_connection_by_ssid()")
-    if not nm.get_connection_by_ssid(ssid):
-        nm.make_hotspot(ssid, device)
+def assure_hotspot(ssid, device, password):
+   log.debug("states: Calling nm.get_connection_by_ssid()")
+   nm.del_connection_by_ssid(ssid)
+   if not nm.get_connection_by_ssid(ssid):
+       nm.make_hotspot(ssid, device, password)
 
 
-def state_monitor():
-    # NetworkManager is crashing on the first call to attach. In two
-    # interface mode, this leaves the hotspot interface with no IP
-    # configuration. Detect this and recover
-    if com_state != 'HOTSPOT':
-        if modemgr.get_mode() == modemgr.MULTI_MODE:
-            log.debug("state_monitor: Calling nm.get_active_ip()")
-            ip = nm.get_active_ip(modemgr.get_state_device('HOTSPOT'))
-            if not ip:
-                log.warn("Hotspot lost IP configuration - resetting")
-                hs_ssid = dns_to_conn(dns_names[0])
-                activate_connection(hs_ssid, 'HOTSPOT')
+def hash_conf():
+    m = hashlib.sha256()
+    with open("/etc/comitup.conf", 'rb') as fp:
+        m.update(fp.read())
 
-    # Keep this periodic task running
-    return True
+    return m.hexdigest()[-4:]
 
 
-def init_states(hosts, callbacks):
+def is_hotspot_current(connection):
+    hs_filename = nm.get_connection_settings(connection)['connection']['id']
+
+    hs_hash = hs_filename[-4:]
+
+    cf_hash = hash_conf()
+
+    return hs_hash == cf_hash
+
+
+def init_states(hosts, callbacks, hotspot_pw):
     global hotspot_name
 
     nmmon.init_nmmon()
@@ -348,10 +341,11 @@ def init_states(hosts, callbacks):
         add_state_callback(callback)
 
     hotspot_name = dns_to_conn(hosts[0])
+    assure_hotspot(hotspot_name, modemgr.get_ap_device(), hotspot_pw)
 
-    assure_hotspot(hotspot_name, modemgr.get_ap_device())
-
-    timeout_add(10*1000, state_monitor)
+    # Set an early kick to set CONNECTING mode
+    set_state('HOTSPOT')
+    timeout_add(5*1000, hotspot_timeout, state_id)
 
 
 def add_state_callback(callback):
@@ -367,7 +361,7 @@ if __name__ == '__main__':
 
     log.info("Starting")
 
-    init_states('comitup.local', 'comitup-1111.local')
+    init_states('comitup.local', 'comitup-1111.local', "")
 
     set_state('HOTSPOT')
     # set_state('CONNECTING', candidate_connections())
