@@ -1,13 +1,13 @@
 
-# Copyright (c) 2018 David Steele <dsteele@gmail.com>
+# Copyright (c) 2018-2019 David Steele <dsteele@gmail.com>
 #
-# SPDX-License-Identifier: GPL-2+
+# SPDX-License-Identifier: GPL-2.0-or-later
 # License-Filename: LICENSE
 
 import pytest
 import urllib
 
-from mock import patch, call, Mock
+from mock import Mock
 
 from web import comitupweb
 
@@ -21,7 +21,7 @@ ssid_list = [
 
 @pytest.fixture
 def app():
-    app = comitupweb.create_app()
+    app = comitupweb.create_app(Mock())
     app.debug = True
     app.testing = True
     return app
@@ -44,25 +44,12 @@ def test_webapp_index(app, ssid, monkeypatch):
     response = app.test_client().get('/')
     index_text = response.get_data().decode()
 
-    assert ssid + "</a>" in index_text
+    assert ssid + "</button>" in index_text
     assert "ssid=" + urllib.parse.quote(ssid) in index_text
 
 
 @pytest.mark.parametrize('ssid', ssid_list)
-def test_webapp_confirm(app, ssid, monkeypatch):
-    quoted_ssid = urllib.parse.quote(ssid)
-    url = "confirm?ssid={}&encrypted=encrypted".format(quoted_ssid)
-
-    response = app.test_client().get(url)
-    index_text = response.get_data().decode()
-
-    assert "to " + ssid in index_text
-    assert "value=\"" + urllib.parse.quote(ssid) in index_text
-
-
-
-@pytest.mark.parametrize('ssid', ssid_list)
-def test_webapp_confirm(app, ssid, monkeypatch):
+def test_webapp_connect(app, ssid, monkeypatch):
     monkeypatch.setattr('web.comitupweb.Process', Mock())
 
     data = {
