@@ -1,7 +1,7 @@
 #!/usr/bin/python3
-# Copyright (c) 2017 David Steele <dsteele@gmail.com>
+# Copyright (c) 2017-2019 David Steele <dsteele@gmail.com>
 #
-# SPDX-License-Identifier: GPL-2+
+# SPDX-License-Identifier: GPL-2.0-or-later
 # License-Filename: LICENSE
 
 #
@@ -11,20 +11,41 @@
 # or later
 #
 
-import os
-import time
+from flask import Flask, render_template, request, send_from_directory,\
+                  redirect, abort
+import logging
+from logging.handlers import TimedRotatingFileHandler
 from multiprocessing import Process
-import urllib
-import base64
-from flask import Flask, render_template, request
-
 import sys
+import time
+import urllib
+
 sys.path.append('.')
 sys.path.append('..')
 
 from comitup import client as ciu                 # noqa
 
 ciu_client = None
+LOG_PATH = "/var/log/comitup-web.log"
+
+
+def deflog():
+    log = logging.getLogger('comitup_web')
+    log.setLevel(logging.INFO)
+    handler = TimedRotatingFileHandler(
+                LOG_PATH,
+                encoding='utf=8',
+                when='D',
+                interval=7,
+                backupCount=8,
+              )
+    fmtr = logging.Formatter(
+            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
+           )
+    handler.setFormatter(fmtr)
+    log.addHandler(handler)
+
+    return log
 
 
 def do_connect(ssid, password):
@@ -32,7 +53,7 @@ def do_connect(ssid, password):
     ciu_client.ciu_connect(ssid, password)
 
 
-def create_app():
+def create_app(log):
     app = Flask(__name__)
 
     @app.route("/")
@@ -40,8 +61,16 @@ def create_app():
         points = ciu_client.ciu_points()
         for point in points:
             point['ssid_encoded'] = urllib.parse.quote(point['ssid'])
+        log.info("index.html - {} points".format(len(points)))
         return render_template("index.html", points=points)
 
+    @app.route('/js/<path:path>')
+    def send_js(path):
+        return send_from_directory('templates/js', path)
+
+    @app.route('/css/<path:path>')
+    def send_css(path):
+        return send_from_directory('templates/css', path)
 
     @app.route("/confirm")
     def confirm():
@@ -51,6 +80,8 @@ def create_app():
 
         mode = ciu_client.ciu_info()['imode']
 
+        log.info("confirm.html - ssid {0}, mode {1}".format(ssid, mode))
+
         return render_template(
                                 "confirm.html",
                                 ssid=ssid,
@@ -59,7 +90,6 @@ def create_app():
                                 mode=mode,
                                 )
 
-
     @app.route("/connect", methods=['POST'])
     def connect():
         ssid = urllib.parse.unquote(request.form["ssid"])
@@ -68,19 +98,36 @@ def create_app():
         p = Process(target=do_connect, args=(ssid, password))
         p.start()
 
-        return render_template("connect.html",
-                                ssid=ssid,
-                                password=password,
-                              )
+        log.info("connect.html - ssid {0}".format(ssid))
+        return render_template(
+                "connect.html",
+                ssid=ssid,
+                password=password,
+                )
+
+    @app.route("/img/favicon.ico")
+    def favicon(path):
+        log.info("Returning 404 for favicon request")
+        abort(404)
+
+    @app.route("/<path:path>")
+    def catch_all(path):
+        return redirect("http://10.42.0.1/", code=302)
 
     return app
 
 
 def main():
+    log = deflog()
+    log.info("Starting comitup-web")
+
     global ciu_client
     ciu_client = ciu.CiuClient()
 
-    app = create_app()
+    ciu_client.ciu_state()
+    ciu_client.ciu_points()
+
+    app = create_app(log)
     app.run(host="0.0.0.0", port=80, debug=True, threaded=True)
 
 
